@@ -21,25 +21,26 @@ opt_sr1 <- optimize_theta(funlist,1.5,control = default_control(method = "SR1"))
 opt_trust <- optimize_theta(funlist,1.5,control = default_control(method = "trust"))
 opt_bfgs <- optimize_theta(funlist,1.5,control = default_control(method = "BFGS"))
 
+norm_sparse_1 <- normalize_logpost(opt_sparsetrust,1,1)
 norm_sparse_3 <- normalize_logpost(opt_sparsetrust,3,1)
 norm_sparse_5 <- normalize_logpost(opt_sparsetrust,5,1)
 norm_sparse_7 <- normalize_logpost(opt_sparsetrust,7,1)
 
+norm_trust_1 <- normalize_logpost(opt_trust,1,1)
 norm_trust_3 <- normalize_logpost(opt_trust,3,1)
 norm_trust_5 <- normalize_logpost(opt_trust,5,1)
 norm_trust_7 <- normalize_logpost(opt_trust,7,1)
 
+norm_bfgs_1 <- normalize_logpost(opt_bfgs,1,1)
 norm_bfgs_3 <- normalize_logpost(opt_bfgs,3,1)
 norm_bfgs_5 <- normalize_logpost(opt_bfgs,5,1)
 norm_bfgs_7 <- normalize_logpost(opt_bfgs,7,1)
 
+norm_sr1_1 <- normalize_logpost(opt_sr1,1,1)
 norm_sr1_3 <- normalize_logpost(opt_sr1,3,1)
 norm_sr1_5 <- normalize_logpost(opt_sr1,5,1)
 norm_sr1_7 <- normalize_logpost(opt_sr1,7,1)
 
-
-
-
 margpost_3 <- marginal_posterior(opt_sparsetrust,3,1)
 pdfwithtrans <- compute_pdf_and_cdf(
   margpost_3,
@@ -88,18 +89,22 @@ opt_trust_2d <- optimize_theta(funlist2d,c(1.5,1.5),control = default_control(me
 opt_sr1_2d <- optimize_theta(funlist2d,c(1.5,1.5),control = default_control(method = "SR1"))
 opt_bfgs_2d <- optimize_theta(funlist2d,c(1.5,1.5),control = default_control(method = "BFGS"))
 
+norm_sparse_2d_1 <- normalize_logpost(opt_sparsetrust_2d,1,1)
 norm_sparse_2d_3 <- normalize_logpost(opt_sparsetrust_2d,3,1)
 norm_sparse_2d_5 <- normalize_logpost(opt_sparsetrust_2d,5,1)
 norm_sparse_2d_7 <- normalize_logpost(opt_sparsetrust_2d,7,1)
 
+norm_trust_2d_1 <- normalize_logpost(opt_trust_2d,1,1)
 norm_trust_2d_3 <- normalize_logpost(opt_trust_2d,3,1)
 norm_trust_2d_5 <- normalize_logpost(opt_trust_2d,5,1)
 norm_trust_2d_7 <- normalize_logpost(opt_trust_2d,7,1)
 
+norm_bfgs_2d_1 <- normalize_logpost(opt_bfgs_2d,1,1)
 norm_bfgs_2d_3 <- normalize_logpost(opt_bfgs_2d,3,1)
 norm_bfgs_2d_5 <- normalize_logpost(opt_bfgs_2d,5,1)
 norm_bfgs_2d_7 <- normalize_logpost(opt_bfgs_2d,7,1)
 
+norm_sr1_2d_1 <- normalize_logpost(opt_sr1_2d,1,1)
 norm_sr1_2d_3 <- normalize_logpost(opt_sr1_2d,3,1)
 norm_sr1_2d_5 <- normalize_logpost(opt_sr1_2d,5,1)
 norm_sr1_2d_7 <- normalize_logpost(opt_sr1_2d,7,1)
@@ -253,18 +258,22 @@ opt_trust_3d <- optimize_theta(funlist3d,c(1.5,1.5,1.5),control = default_contro
 opt_sr1_3d <- optimize_theta(funlist3d,c(1.5,1.5,1.5),control = default_control(method = "SR1"))
 opt_bfgs_3d <- optimize_theta(funlist3d,c(1.5,1.5,1.5),control = default_control(method = "BFGS"))
 
+norm_sparse_3d_1 <- normalize_logpost(opt_sparsetrust_3d,1,1)
 norm_sparse_3d_3 <- normalize_logpost(opt_sparsetrust_3d,3,1)
 norm_sparse_3d_5 <- normalize_logpost(opt_sparsetrust_3d,5,1)
 norm_sparse_3d_7 <- normalize_logpost(opt_sparsetrust_3d,7,1)
 
+norm_trust_3d_1 <- normalize_logpost(opt_trust_3d,1,1)
 norm_trust_3d_3 <- normalize_logpost(opt_trust_3d,3,1)
 norm_trust_3d_5 <- normalize_logpost(opt_trust_3d,5,1)
 norm_trust_3d_7 <- normalize_logpost(opt_trust_3d,7,1)
 
+norm_bfgs_3d_1 <- normalize_logpost(opt_bfgs_3d,1,1)
 norm_bfgs_3d_3 <- normalize_logpost(opt_bfgs_3d,3,1)
 norm_bfgs_3d_5 <- normalize_logpost(opt_bfgs_3d,5,1)
 norm_bfgs_3d_7 <- normalize_logpost(opt_bfgs_3d,7,1)
 
+norm_sr1_3d_1 <- normalize_logpost(opt_sr1_3d,1,1)
 norm_sr1_3d_3 <- normalize_logpost(opt_sr1_3d,3,1)
 norm_sr1_3d_5 <- normalize_logpost(opt_sr1_3d,5,1)
 norm_sr1_3d_7 <- normalize_logpost(opt_sr1_3d,7,1)
@@ -403,7 +412,7 @@ logaghq <- function(n) {
     gr = function(x) numDeriv::grad(logint1,x,n=n),
     he = function(x) numDeriv::hessian(logint1,x,n=n)
   )
-  aghq::laplace_approximation(ff,n)$lognormconst
+  get_log_normconst(aghq::laplace_approximation(ff,n))
 }
 logstirling <- function(n) (1/2)*log(2*pi) + (1/2)*log(n) + n*(log(n) - 1)
 la5 <- round(logaghq(5),11)
@@ -712,3 +721,186 @@ thesummary_correct <- summary(thequadrature_correct)
 truemean_correct <- digamma(c(sum(y_correct[1:5]) + 1,sum(y_correct[6:10]) + 1)) - log(c(5 + 1,5+1))
 truesd_correct <- sqrt(trigamma(c(sum(y_correct[1:5])+1,sum(y_correct[6:10]) + 1)))
 
+## Nested ##
+quadtable_p1_k1_prod <- get_quadtable(1,1)
+quadtable_p2_k1_prod <- get_quadtable(2,1)
+quadtable_p3_k1_prod <- get_quadtable(3,1)
+quadtable_p4_k1_prod <- get_quadtable(4,1)
+quadtable_p5_k1_prod <- get_quadtable(5,1)
+
+quadtable_p1_k3_prod <- get_quadtable(1,3)
+quadtable_p2_k3_prod <- get_quadtable(2,3)
+quadtable_p3_k3_prod <- get_quadtable(3,3)
+quadtable_p4_k3_prod <- get_quadtable(4,3)
+quadtable_p5_k3_prod <- get_quadtable(5,3)
+
+quadtable_p1_k5_prod <- get_quadtable(1,5)
+quadtable_p2_k5_prod <- get_quadtable(2,5)
+quadtable_p3_k5_prod <- get_quadtable(3,5)
+quadtable_p4_k5_prod <- get_quadtable(4,5)
+quadtable_p5_k5_prod <- get_quadtable(5,5)
+
+quadtable_p1_k1_sparse <- get_quadtable(1,1,'sparse')
+quadtable_p2_k1_sparse <- get_quadtable(2,1,'sparse')
+quadtable_p3_k1_sparse <- get_quadtable(3,1,'sparse')
+quadtable_p4_k1_sparse <- get_quadtable(4,1,'sparse')
+quadtable_p5_k1_sparse <- get_quadtable(5,1,'sparse')
+
+quadtable_p1_k3_sparse <- get_quadtable(1,3,'sparse')
+quadtable_p2_k3_sparse <- get_quadtable(2,3,'sparse')
+quadtable_p3_k3_sparse <- get_quadtable(3,3,'sparse')
+quadtable_p4_k3_sparse <- get_quadtable(4,3,'sparse')
+quadtable_p5_k3_sparse <- get_quadtable(5,3,'sparse')
+
+quadtable_p1_k5_sparse <- get_quadtable(1,5,'sparse')
+quadtable_p2_k5_sparse <- get_quadtable(2,5,'sparse')
+quadtable_p3_k5_sparse <- get_quadtable(3,5,'sparse')
+quadtable_p4_k5_sparse <- get_quadtable(4,5,'sparse')
+quadtable_p5_k5_sparse <- get_quadtable(5,5,'sparse')
+
+nestedoptlist1 <- list(
+  fn = function(x) sum(dnorm(x,log=TRUE)),
+  mode = rep(0,1)
+)
+nestedoptlist2 <- list(
+  fn = function(x) sum(dnorm(x,log=TRUE)),
+  mode = rep(0,2)
+)
+nestedoptlist3 <- list(
+  fn = function(x) sum(dnorm(x,log=TRUE)),
+  mode = rep(0,3)
+)
+nestedoptlist4 <- list(
+  fn = function(x) sum(dnorm(x,log=TRUE)),
+  mode = rep(0,4)
+)
+nestedoptlist5 <- list(
+  fn = function(x) sum(dnorm(x,log=TRUE)),
+  mode = rep(0,5)
+)
+
+nq_p1_k1_prod <- nested_quadrature(nestedoptlist1,1,'product')
+nq_p2_k1_prod <- nested_quadrature(nestedoptlist2,1,'product')
+nq_p3_k1_prod <- nested_quadrature(nestedoptlist3,1,'product')
+nq_p4_k1_prod <- nested_quadrature(nestedoptlist4,1,'product')
+nq_p5_k1_prod <- nested_quadrature(nestedoptlist5,1,'product')
+
+nq_p1_k3_prod <- nested_quadrature(nestedoptlist1,3,'product')
+nq_p2_k3_prod <- nested_quadrature(nestedoptlist2,3,'product')
+nq_p3_k3_prod <- nested_quadrature(nestedoptlist3,3,'product')
+nq_p4_k3_prod <- nested_quadrature(nestedoptlist4,3,'product')
+nq_p5_k3_prod <- nested_quadrature(nestedoptlist5,3,'product')
+
+nq_p1_k5_prod <- nested_quadrature(nestedoptlist1,5,'product')
+nq_p2_k5_prod <- nested_quadrature(nestedoptlist2,5,'product')
+nq_p3_k5_prod <- nested_quadrature(nestedoptlist3,5,'product')
+nq_p4_k5_prod <- nested_quadrature(nestedoptlist4,5,'product')
+nq_p5_k5_prod <- nested_quadrature(nestedoptlist5,5,'product')
+
+nq_p1_k1_sparse <- nested_quadrature(nestedoptlist1,1,'sparse')
+nq_p2_k1_sparse <- nested_quadrature(nestedoptlist2,1,'sparse')
+nq_p3_k1_sparse <- nested_quadrature(nestedoptlist3,1,'sparse')
+nq_p4_k1_sparse <- nested_quadrature(nestedoptlist4,1,'sparse')
+nq_p5_k1_sparse <- nested_quadrature(nestedoptlist5,1,'sparse')
+
+nq_p1_k3_sparse <- nested_quadrature(nestedoptlist1,3,'sparse')
+nq_p2_k3_sparse <- nested_quadrature(nestedoptlist2,3,'sparse')
+nq_p3_k3_sparse <- nested_quadrature(nestedoptlist3,3,'sparse')
+nq_p4_k3_sparse <- nested_quadrature(nestedoptlist4,3,'sparse')
+nq_p5_k3_sparse <- nested_quadrature(nestedoptlist5,3,'sparse')
+
+nq_p1_k5_sparse <- nested_quadrature(nestedoptlist1,5,'sparse')
+nq_p2_k5_sparse <- nested_quadrature(nestedoptlist2,5,'sparse')
+nq_p3_k5_sparse <- nested_quadrature(nestedoptlist3,5,'sparse')
+nq_p4_k5_sparse <- nested_quadrature(nestedoptlist4,5,'sparse')
+nq_p5_k5_sparse <- nested_quadrature(nestedoptlist5,5,'sparse')
+
+# Adaptive nested quadrature
+man1 <- 1
+man2 <- 1:2
+man3 <- 1:3
+man4 <- 1:4
+man5 <- 1:5
+
+Han1 <- as.matrix(2)
+Han2 <- Matrix::crossprod(matrix(rnorm(2^2),nrow=2))
+Han3 <- Matrix::crossprod(matrix(rnorm(3^2),nrow=3))
+Han4 <- Matrix::crossprod(matrix(rnorm(4^2),nrow=4))
+Han5 <- Matrix::crossprod(matrix(rnorm(5^2),nrow=5))
+
+
+dmnorm <- function(x,man,Han) { # multivariate normal
+  p <- length(man)
+  if (p == 1) return(dnorm(x,man,1/sqrt(Han),log = TRUE))
+  up <- -0.5 * as.numeric(crossprod(x-man,crossprod(Han,x-man)))
+  down <- -0.5*p*log(2*pi) + 0.5*as.numeric(determinant(Han,logarithm = TRUE)$modulus)
+  up + down
+}
+adaptivenestedoptlist1 <- list(
+  fn = function(x) dmnorm(x,man1,Han1),
+  mode = man1,
+  hessian = as.matrix(Han1)
+)
+adaptivenestedoptlist2 <- list(
+  fn = function(x) dmnorm(x,man2,Han2),
+  mode = man2,
+  hessian = Han2
+)
+adaptivenestedoptlist3 <- list(
+  fn = function(x) dmnorm(x,man3,Han3),
+  mode = man3,
+  hessian = Han3
+)
+adaptivenestedoptlist4 <- list(
+  fn = function(x) dmnorm(x,man4,Han4),
+  mode = man4,
+  hessian = Han4
+)
+adaptivenestedoptlist5 <- list(
+  fn = function(x) dmnorm(x,man5,Han5),
+  mode = man5,
+  hessian = Han5
+)
+
+
+
+anq_p1_k1_prod <- adaptive_nested_quadrature(adaptivenestedoptlist1,1,'product')
+anq_p2_k1_prod <- adaptive_nested_quadrature(adaptivenestedoptlist2,1,'product')
+anq_p3_k1_prod <- adaptive_nested_quadrature(adaptivenestedoptlist3,1,'product')
+anq_p4_k1_prod <- adaptive_nested_quadrature(adaptivenestedoptlist4,1,'product')
+anq_p5_k1_prod <- adaptive_nested_quadrature(adaptivenestedoptlist5,1,'product')
+
+anq_p1_k3_prod <- adaptive_nested_quadrature(adaptivenestedoptlist1,3,'product')
+anq_p2_k3_prod <- adaptive_nested_quadrature(adaptivenestedoptlist2,3,'product')
+anq_p3_k3_prod <- adaptive_nested_quadrature(adaptivenestedoptlist3,3,'product')
+anq_p4_k3_prod <- adaptive_nested_quadrature(adaptivenestedoptlist4,3,'product')
+anq_p5_k3_prod <- adaptive_nested_quadrature(adaptivenestedoptlist5,3,'product')
+
+anq_p1_k5_prod <- adaptive_nested_quadrature(adaptivenestedoptlist1,5,'product')
+anq_p2_k5_prod <- adaptive_nested_quadrature(adaptivenestedoptlist2,5,'product')
+anq_p3_k5_prod <- adaptive_nested_quadrature(adaptivenestedoptlist3,5,'product')
+anq_p4_k5_prod <- adaptive_nested_quadrature(adaptivenestedoptlist4,5,'product')
+anq_p5_k5_prod <- adaptive_nested_quadrature(adaptivenestedoptlist5,5,'product')
+
+anq_p1_k1_sparse <- adaptive_nested_quadrature(adaptivenestedoptlist1,1,'sparse')
+anq_p2_k1_sparse <- adaptive_nested_quadrature(adaptivenestedoptlist2,1,'sparse')
+anq_p3_k1_sparse <- adaptive_nested_quadrature(adaptivenestedoptlist3,1,'sparse')
+anq_p4_k1_sparse <- adaptive_nested_quadrature(adaptivenestedoptlist4,1,'sparse')
+anq_p5_k1_sparse <- adaptive_nested_quadrature(adaptivenestedoptlist5,1,'sparse')
+
+anq_p1_k3_sparse <- adaptive_nested_quadrature(adaptivenestedoptlist1,3,'sparse')
+anq_p2_k3_sparse <- adaptive_nested_quadrature(adaptivenestedoptlist2,3,'sparse')
+anq_p3_k3_sparse <- adaptive_nested_quadrature(adaptivenestedoptlist3,3,'sparse')
+anq_p4_k3_sparse <- adaptive_nested_quadrature(adaptivenestedoptlist4,3,'sparse')
+anq_p5_k3_sparse <- adaptive_nested_quadrature(adaptivenestedoptlist5,3,'sparse')
+
+anq_p1_k5_sparse <- adaptive_nested_quadrature(adaptivenestedoptlist1,5,'sparse')
+anq_p2_k5_sparse <- adaptive_nested_quadrature(adaptivenestedoptlist2,5,'sparse')
+anq_p3_k5_sparse <- adaptive_nested_quadrature(adaptivenestedoptlist3,5,'sparse')
+anq_p4_k5_sparse <- adaptive_nested_quadrature(adaptivenestedoptlist4,5,'sparse')
+anq_p5_k5_sparse <- adaptive_nested_quadrature(adaptivenestedoptlist5,5,'sparse')
+
+
+
+
+
